import axios from 'axios';
import { Notification, MessageBox, Message, Loading } from 'element-ui';
import store from '@/store';
import { getToken, getTenantid } from '@/utils/auth';
import errorCode from '@/utils/errorCode';
import { tansParams, blobValidate } from "@/utils/ruoyi";
import cache from '@/plugins/cache';
import { saveAs } from 'file-saver';

let downloadLoadingInstance;
// 是否显示重新登录
export let isRelogin = { show: false };

axios.defaults.headers['Content-Type'] = 'application/json;charset=utf-8';
// 创建axios实例
const service = axios.create({
  // axios中请求配置有baseURL选项，表示请求URL公共部分
  baseURL: process.env.VUE_APP_BASE_API,
  // 超时
  timeout: 20000
});

// request拦截器
service.interceptors.request.use(config => {
  // 是否需要设置 token
  const isToken = (config.headers || {}).isToken === false;
  // 是否需要防止数据重复提交
  const isRepeatSubmit = (config.headers || {}).repeatSubmit === false;
  if (getToken() && !isToken) {
    config.headers['Authorization'] = 'Bearer ' + getToken(); // 让每个请求携带自定义token 请根据实际情况自行修改
    config.headers['tenantid'] = getTenantid(); //携带租户id
  }
  // get请求映射params参数
  if (config.method === 'get' && config.params) {
    let url = config.url + '?' + tansParams(config.params);
    url = url.slice(0, -1);
    config.params = {};
    config.url = url;
  }
  // cancelPending(config)
  // config.cancelToken = new CancelToken(res => {
  //   pending.push({'UrlPath': config.url, 'Cancel': res})
  // })
  // if (!isRepeatSubmit && (config.method === 'post' || config.method === 'put')) {
  //   const requestObj = {
  //     url: config.url,
  //     data: typeof config.data === 'object' ? JSON.stringify(config.data) : config.data,
  //     time: new Date().getTime()
  //   }
  //   const sessionObj = cache.session.getJSON('sessionObj')
  //   if (sessionObj === undefined || sessionObj === null || sessionObj === '') {
  //     cache.session.setJSON('sessionObj', requestObj)
  //   } else {
  //     const s_url = sessionObj.url;                  // 请求地址
  //     const s_data = sessionObj.data;                // 请求数据
  //     const s_time = sessionObj.time;                // 请求时间
  //     const interval = 200;                         // 间隔时间(ms)，小于此时间视为重复提交
  //     if (s_data === requestObj.data && requestObj.time - s_time < interval && s_url === requestObj.url) {
  //       const message = '数据正在处理，请勿重复提交';
  //       console.warn(`[${s_url}]: ` + message)
  //       return Promise.reject(new Error(message))
  //     } else {
  //       cache.session.setJSON('sessionObj', requestObj)
  //     }
  //   }
  // }
  return config;
}, error => {
  console.log(error);
  if (error.message.indexOf('timeout') < 0) { //超时报错不提示
    Promise.reject(error);
  }
});

//频繁发送请求时，返回慢的数据覆盖了返回快的数据
let pending = [];
let CancelToken = axios.CancelToken;
let cancelPending = (config) => {
  pending.forEach((item, index) => {
    if (config) {
      // if (item.UrlPath === '/combine/info/memberList' || item.UrlPath === '/combine/info/businessList' || item.UrlPath === '/combine/info/bidPage') {
      if (item.UrlPath === config.url && ['/combine/info/memberList', '/combine/info/businessList', '/combine/info/bidPage'].includes(config.url)) {
        item.Cancel(); // 取消请求
        pending.splice(index, 1); // 移除当前请求记录
      };
    } else {
      item.Cancel(); // 取消请求
      pending.splice(index, 1); // 移除当前请求记录
    }
  });
};

// 响应拦截器
service.interceptors.response.use(res => {
  const { config, headers } = res;
  const apiList = ['/login', '/system/user/profile/updatePwd'];  //需要提示错误的接口
  // 未设置状态码则默认成功状态
  const code = res.data.code || 200;
  // 获取错误信息
  const msg = errorCode[code] || res.data.msg || errorCode['default'];
  // 文件下载
  if (config.isFile) {
    return {
      data: res.data,
      fileName: headers["content-disposition"]
    };
  }
  // 二进制数据则直接返回
  if (res.request.responseType === 'blob' || res.request.responseType === 'arraybuffer') {
    return res.data;
  }
  if (code === 401) {
    if (!isRelogin.show) {
      isRelogin.show = true;
      MessageBox.confirm('登录状态已过期，您可以继续留在该页面，或者重新登录', '系统提示', { confirmButtonText: '重新登录', cancelButtonText: '取消', type: 'warning' }).then(() => {
        isRelogin.show = false;
        store.dispatch('LogOut').then(() => {
          sessionStorage.removeItem('views');
          location.href = '/index/gys';
        });
      }).catch(() => {
        isRelogin.show = false;
      });
    }
    return Promise.reject('无效的会话，或者会话已过期，请重新登录。');
  } else if (code === 500) {
    Message({ message: msg, type: 'error' });
    return Promise.reject(new Error(msg));
  } else if (code === 601) {
    Message({ message: msg, type: 'warning' });
    return Promise.reject('error');
  } else if (code !== 200) {
    Notification.error({ title: msg });
    return Promise.reject('error');
  } else {
    // cancelPending(res.config)
    return res.data;
  }
}, error => {
  console.log(error);
  console.log('err' + error);
  let { message } = error;
  /*if (message == "Network Error") {
    message = "后端接口连接异常";
  } else if (message.includes("timeout")) {
    message = "系统接口请求超时";
  } else if (message.includes("Request failed with status code")) {
    message = "系统接口" + message.substr(message.length - 3) + "异常";
  }*/
  if (message == '数据正在处理，请勿重复提交') { //重复提交，提示样式特殊处理
    Message({ message: message, type: 'warning', duration: 5 * 1000 });
  } else {
    // Message({ message: message, type: 'error', duration: 5 * 1000 })
  }

  if (error.message.indexOf('timeout') < 0) { //超时报错不提示
    return Promise.reject(error);
  }
  // return Promise.reject(error)
}
);

// 通用下载方法
export function download(url, params, filename, config) {
  downloadLoadingInstance = Loading.service({ text: "正在下载数据，请稍候", spinner: "el-icon-loading", background: "rgba(0, 0, 0, 0.7)", });
  return service.post(url, params, {
    transformRequest: [(params) => { return tansParams(params); }],
    headers: { 'Content-Type': 'application/x-www-form-urlencoded' },
    responseType: 'blob',
    ...config
  }).then(async (data) => {
    const isBlob = blobValidate(data);
    if (isBlob) {
      const blob = new Blob([data]);
      saveAs(blob, filename);
    } else {
      const resText = await data.text();
      const rspObj = JSON.parse(resText);
      const errMsg = errorCode[rspObj.code] || rspObj.msg || errorCode['default'];
      Message.error(errMsg);
    }
    downloadLoadingInstance.close();
  }).catch((r) => {
    console.error(r);
    Message.error('下载文件出现错误，请联系管理员！');
    downloadLoadingInstance.close();
  });
}

export default service;
